package com.company.jaxws.handler.cardservice.client;

import javax.xml.ws.BindingProvider;

import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.ws.Binding;
import javax.xml.ws.Service;
import javax.xml.ws.Dispatch;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.handler.HandlerResolver;

import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.namespace.QName;

import com.company.jaxws.handler.cardservice.client.CreditCardUtil;
import com.company.jaxws.handler.cardservice.common.AuthenticationHandlerConstants;
import com.company.jaxws.handler.cardservice.common.ClientAuthenticationSOAPHandler;

/**
 * Client for Credit Card Service
 */
public class CreditCardServiceClient {

	private final static String requestXMLFile = "./conf/AuthReq.xml";
	private final static String requestSOAPXMLFile = "./conf/AuthSOAPReq.xml";

	CreditCardUtil util = null;

	QName serviceName = null;
	QName portName = null;
	URL wsdlLoc = null;

	HandlerResolver ccResolver = null;

	public CreditCardServiceClient(String url) {
		try {
			serviceName = new QName(
					"http://cardservice.handler.jaxws.company.com/service",
					"CardService");
			wsdlLoc = new URL(url + "?wsdl");
			portName = new QName(
					"http://cardservice.handler.jaxws.company.com/service",
					"CardServicePort");
			ccResolver = new CardServiceHandlerResolver();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void authorizePayment(String url, Service.Mode mode)
			throws Exception {
		String xmlString = null;
		try {
			Service service = Service.create(wsdlLoc, serviceName);
			//service.addPort(portName, SOAPBinding.SOAP11HTTP_BINDING, url);
			service.setHandlerResolver(ccResolver);

			if (mode.equals(Service.Mode.PAYLOAD)) {

				Dispatch<Source> disp = service.createDispatch(portName,
						Source.class, mode);

				Map<String, Object> requestCtx = ((BindingProvider) disp)
						.getRequestContext();
				requestCtx.put(AuthenticationHandlerConstants.REQUEST_USERID,
						"yyang");
				requestCtx.put(AuthenticationHandlerConstants.REQUEST_PASSWORD,
						"mypassword");

				xmlString = CreditCardUtil.readStringFromFile(requestXMLFile);
				Source xmlSource = (Source) new StreamSource(new StringReader(
						xmlString));

				Source response = disp.invoke(xmlSource);

				System.out.println(CreditCardUtil.sourceToXMLString(response));
			}
			
			if (mode.equals(Service.Mode.MESSAGE)) {
				Dispatch<SOAPMessage> disp = service.createDispatch(portName,
						SOAPMessage.class, mode);

				Map<String, Object> requestCtx = ((BindingProvider) disp)
						.getRequestContext();
				requestCtx.put(AuthenticationHandlerConstants.REQUEST_USERID,
						"yyang");
				requestCtx.put(AuthenticationHandlerConstants.REQUEST_PASSWORD,
						"mypassword");

				Binding cardServiceBinding = ((BindingProvider) disp).getBinding();				
				ClientAuthenticationSOAPHandler authnHandler = new ClientAuthenticationSOAPHandler();				
				List<Handler> handlerChain = new ArrayList<Handler>();				
				handlerChain.add(authnHandler);				
				cardServiceBinding.setHandlerChain(handlerChain);
				
				xmlString = CreditCardUtil
						.readStringFromFile(requestSOAPXMLFile);

				MessageFactory factory = MessageFactory.newInstance();
				SOAPMessage message = factory.createMessage();
				message.getSOAPPart().setContent(
						(Source) new StreamSource(new StringReader(xmlString)));
				message.saveChanges();

				SOAPMessage response = disp.invoke(message);

				SOAPPart sp = response.getSOAPPart();
				Source resp = sp.getContent();
				System.out.println(CreditCardUtil.sourceToXMLString(resp));
			}
		} catch (Exception e) {
			System.out.println("Card Service Exception: " + e.getMessage());
			e.printStackTrace();
		}
	}

	public static void main(String[] args) throws Exception {
		CreditCardServiceClient cctest = new CreditCardServiceClient(args[0]);

		try {
			System.out.println("Calling Credit Card Service in " + args[1]
					+ " Mode ------ ");

			System.out.println("Response from Credit Card Service:");
			cctest.authorizePayment(args[0], Service.Mode.valueOf(args[1]));

		} catch (Exception e) {
			System.out.println("CardServiceException: " + e.getMessage());
			e.printStackTrace();
		}
	}
}
