package com.company.jaxws.handler.cardservice;

import java.util.Date;
import java.util.Random;

import javax.annotation.Resource;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import com.company.jaxws.handler.cardservice.service.CardServiceException;
import com.company.jaxws.handler.cardservice.service.CardServicePortType;
import com.company.jaxws.handler.cardservice.creditcard.AuthorizationRequest;
import com.company.jaxws.handler.cardservice.creditcard.AuthorizationStatus;
import com.company.jaxws.handler.cardservice.creditcard.CardUser;
import com.company.jaxws.handler.cardservice.creditcard.CreditCard;
import com.company.jaxws.handler.cardservice.exception.FaultInfo;

@javax.jws.WebService(targetNamespace = "http://cardservice.handler.jaxws.company.com/service",
				endpointInterface = "com.company.jaxws.handler.cardservice.service.CardServicePortType")
public class CreditCardServiceImpl implements CardServicePortType {
	private static Random rdm;
    private String authorizationToken;
    private boolean authorized;
    private int errorCode;
    
    static {
    	rdm = new Random();
    }
    
    @Resource
    WebServiceContext wsContext;
 
	public AuthorizationStatus authorizePayment(AuthorizationRequest request) throws CardServiceException {
		try {
			MessageContext msgContext = wsContext.getMessageContext();			
			Boolean authnStatus = (Boolean)msgContext.get("authnStatus");
			
			CreditCard card = request.getCreditCard();
			CardUser user = request.getCardUser();
			
			StringBuffer cardInfo = new StringBuffer();
			System.out.println("------------------------------------");
			System.out.println("Messages from the Service Implementation:");
			System.out.println(" ");
			
			cardInfo.append("********************************************** \n");
			cardInfo.append("Received Authorization Request on " + new Date() + " for Credit Card: \n");
			cardInfo.append("Card Holder: " + user.getLastName() + " " + user.getFirstName() + "\n");
			cardInfo.append("Card Number: " + card.getCardNumber() + "\n");
			cardInfo.append("Card CCV Number: " + card.getCcvNumber() + "\n");
			cardInfo.append("Billing Address: " + card.getBillingAddress().toString() + "\n");
			if (authnStatus != null) cardInfo.append("Authentication Status as Reported by ServiceAuthentricationSOAPHandler is:" + authnStatus.toString() + "\n");
			cardInfo.append("********************************************** \n");
			System.out.println(cardInfo.toString());
			
			// Emulate charge authorization
			authorizeCharge();			

			StringBuffer statusInfo = new StringBuffer();
			
			statusInfo.append("********************************************** \n");
			statusInfo.append("Respond with Authorization Result for the Received Credit Card: \n");
			statusInfo.append("\n");
			statusInfo.append("Authorization Token: " + authorizationToken + "\n");
			statusInfo.append("Is Authorized? :" + authorized + "\n");
			statusInfo.append("Error Code: " + errorCode + "\n");
			statusInfo.append("********************************************** \n");
			System.out.println(statusInfo.toString());
			
			System.out.println("------------------------------------");
			
			// Create and populate authorization status object
			AuthorizationStatus status = new AuthorizationStatus();
			status.setAuthorizationToken(authorizationToken);
			status.setAuthorized(authorized);
			status.setErrorCode(errorCode);

			return status;
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Exception while authorizing payment:"
					+ e.getMessage());
			FaultInfo abInfo = new FaultInfo();
			abInfo.setFaultMessage("Error Code:" + errorCode);
			throw new CardServiceException("Authorization Exception",abInfo);
		}
	}
	

  	private void authorizeCharge() {
        authorizationToken = String.valueOf(getRandom(1,-1));          
        errorCode = 0; // error code 0 for passed authorization
        
        authorized = isAuthorized(); // authorized true or false
        
        // If this payment is not authorized, set error code
        if (authorized == false) {
        	authorizationToken = "0";
            errorCode = getRandom(999); // 3 digit error code            
        }
    }

    private boolean isAuthorized() {
        boolean auth = true;        
        if (getRandom(11)%2 > 0){
        	auth = false;
        }        
        return auth;
    }


    //Random utils    
    private int getRandom(int limit) {
        int randNum = rdm.nextInt(limit); // random number 0 to limit
        if(randNum < 0) randNum = randNum*(-1); // only positive        
        return randNum;
    }
    
    private int getRandom(int lowerLimit, int upperLimit) {
        int randNum;
        if(upperLimit <= 0)
            upperLimit = 99999999; // 8 digit number
        do {
            randNum = rdm.nextInt(upperLimit); // random number between limits
        } while(randNum < lowerLimit); // lower is inclusive
        if(randNum < 0) randNum = randNum*(-1); // only positive        
        return randNum;
    }
}
